<?php
/**
 * Copyright (c) 2012 Robin Appelman <icewind@owncloud.com>
 * This file is licensed under the Affero General Public License version 3 or
 * later.
 * See the COPYING-README file.
 */

namespace OC\Files\Storage;

abstract class StreamWrapper extends Common{
        private $stats;
        private $mimetypes=array();

	abstract public function constructUrl($path);

	public function mkdir($path) {
		return mkdir($this->constructUrl($path));
	}

	public function rmdir($path) {
		if($this->file_exists($path)) {
			$success = rmdir($this->constructUrl($path));
			clearstatcache();
			return $success;
		} else {
			return false;
		}
	}

	public function is_dir($path) {
		return $this->stat($path,"is_dir");
	}

	public function opendir($path) {
		return opendir($this->constructUrl($path));
	}

	public function filetype($path) {
                if($this->file_exists($path)){
                        return filetype($this->constructUrl($path)); // new connection
                }
                return false;
	}

	public function isReadable($path) {
               return $this->stat($path,"isReadable");
	}

	public function isUpdatable($path) {
                return $this->stat($path,"isUpdatable");
	}

	public function file_exists($path) {
                return $this->stat($path,"file_exists");
	}

	public function unlink($path) {
		$success = unlink($this->constructUrl($path));
		clearstatcache();
		return $success;
	}

	public function fopen($path, $mode) {
		return fopen($this->constructUrl($path), $mode);
	}

	public function touch($path, $mtime=null) {
		if(is_null($mtime)) {
			$fh = $this->fopen($path, 'a');
			fwrite($fh, '');
			fclose($fh);
		} else {
			return false;//not supported
		}
	}

	public function getFile($path, $target) {
		return copy($this->constructUrl($path), $target);
	}

	public function uploadFile($path, $target) {
		return copy($path, $this->constructUrl($target));
	}

	public function rename($path1, $path2) {
                // Deleting .part if exists.
                if ($path1==$path2.".part") $this->unlink($path2);
                $res=rename($this->constructUrl($path1), $this->constructUrl($path2));
                if ($res) { // On success we need to update the file cache.
                        $deleted_cache=new \OC\Files\Cache\Cache($this->getId());
                        $deleted_cache->remove($path1);
                }
                return $res;
        }

        /**
	 * Aggregator function for file_exists, is_dir, isReadable, isUpdatable and stat calls.
	 *
	 * Stat a file or a folder on a remote storage maybe slow and expensive. This function 
	 * tries to get all possible metadata about a remote object by only one connection.
	 * Results are stored in an array, used as "microcache" for repetitive function calls.
	 *
	 * @param string $path
	 * @param string $stat_type	valid: file_exists, is_dir, isReadable, isUpdatable, stat
	 * @return mixed		array for stat, boolean for others, false if failed.
	 */
        public function stat($path,$stat_type="stat") {
			if(empty($this->stats[$path])){
				// Microcache failed, trying a real query and save results.
				if(file_exists($this->constructUrl($path))){
					$this->stats[$path]["file_exists"]=true;
					$this->stats[$path]["isReadable"]=true;	 // not properly supported
					$this->stats[$path]["isUpdatable"]=true; // not properly supported
					$this->stats[$path]["is_dir"]=is_dir($this->constructUrl($path));
					$stat=stat($this->constructUrl($path));
					// A trick call if mtime doesn't work on root folder (like ftp).
					if(($path=="" || $path=='/') && $stat['mtime']<0) $stat['mtime']=$this->shareMTime();
					$this->stats[$path]["stat"]=$stat;
				}
			}
			return (empty($this->stats[$path][$stat_type])) ? false : $this->stats[$path][$stat_type];
        }

	/**
	 * Getting or guessing MIME type.
	 *
	 * To open a file on a remote storage (to perform a download) to get the MIME type is slow and expensive.
	 * This overriding function examines the extension database first, then looks for metadata in cache content.
	 * When steps above were failed, the original getMimeType function gets called only if downloading for MIME 
	 * is enabled in configuration.
	 *
	 * @param string $path
	 * @return string
	 */
	public function getMimeType($path) {
		if (!$this->file_exists($path)) { // costs a stat call
			return false;
		}
		if ($this->is_dir($path)) { // not a real stat, uses "microcache"
			return 'httpd/unix-directory';
	}
		$mimeType='application/octet-stream';

		// Try to guess the type by the file extension.
		if(strpos($path, '.')) {
			if(!$this->mimetypes || $this->mimetypes != include 'lib/mimetypes.list.php') {
				$this->mimetypes=include 'lib/mimetypes.list.php';
			}
			$extension=strtolower(strrchr(basename($path), "."));
			$extension=substr($extension, 1); // remove leading dot (.)
			$mimeType=(isset($this->mimetypes[$extension]))? $this->mimetypes[$extension]:'application/octet-stream';
			if ($mimeType!='application/octet-stream') {
				return $mimeType; // Success
			}
		}

		// Try from cache.
		$cache = new \OC\Files\Cache\Cache($this);
		$cacheData=$cache->get($path);
		if($cacheData && !$this->hasUpdated($path,$cacheData["mtime"])) { // not a real stat, uses "microcache"
			return $cacheData["mimetype"];
		}

		// Cache failed, continue with download for analysis only if it is enabled.
		if($mimeType!='application/octet-stream' || !\OC_Config::getValue('down4mime',false)) {
			return $mimeType;
		} else {
			return parent::getMimeType($path);
		}
	}

	/**
	 * get the best guess for the modification time of the share
	 */
	private function shareMTime() {
		$dh=$this->opendir('');
		$lastCtime=0;
		while($file=readdir($dh)) {
			if ($file!='.' and $file!='..') {
				$ctime=$this->filemtime($file);
				if ($ctime>$lastCtime) {
					$lastCtime=$ctime;
				}
			}
		}
		return $lastCtime;
	}

}
